"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const emitter = require("ts-emitter");
const ts = require("typescript");
const pattern_matcher_1 = require("./pattern-matcher");
class Collection {
    constructor(collected, root) {
        this._root = root;
        this.collected = collected;
    }
    static fromSource(source) {
        const file = emitter.fromSource(source);
        return new Collection([file]);
    }
    static fromNode(node) {
        return new Collection([node]);
    }
    get root() {
        return this._root || this;
    }
    get rootNode() {
        return this.root.collected[0];
    }
    set rootNode(node) {
        this.root.collected[0] = node;
    }
    find(kind, pattern) {
        const marked = [];
        const visitor = (node) => {
            if (node.kind === kind) {
                if (pattern && pattern_matcher_1.isPatternMatching(node, pattern)) {
                    marked.push(node);
                }
                else if (!pattern) {
                    marked.push(node);
                }
                else {
                    ts.forEachChild(node, visitor);
                }
            }
            else {
                ts.forEachChild(node, visitor);
            }
        };
        this.collected.forEach(node => {
            ts.forEachChild(node, visitor);
        });
        return new Collection(marked, this.root);
    }
    get(fn) {
        const marked = [];
        this.collected.forEach(node => {
            const result = fn(node);
            if (result) {
                marked.push(result);
            }
        });
        return new Collection(marked, this.root);
    }
    filter(fn) {
        const marked = [];
        this.collected.forEach(node => {
            if (fn(node)) {
                marked.push(node);
            }
        });
        return new Collection(marked, this.root);
    }
    replaceWith(fn) {
        const replacer = (context) => (rootNode) => {
            const visitor = (node) => {
                const markedNode = this.collected.find(item => item === node);
                if (markedNode) {
                    const replaced = fn(markedNode);
                    if (replaced !== markedNode) {
                        replaced.original = markedNode;
                        if (replaced.text && markedNode.text) {
                            replaced.newText = replaced.text;
                        }
                    }
                    return replaced;
                }
                return ts.visitEachChild(node, visitor, context);
            };
            return ts.visitNode(rootNode, visitor);
        };
        this.rootNode = ts.transform(this.rootNode, [replacer]).transformed[0];
        return this;
    }
    forEach(fn) {
        this.collected.forEach(node => fn(node));
        return this;
    }
    size() {
        return this.collected.length;
    }
    toSource() {
        if (this.rootNode.kind !== ts.SyntaxKind.SourceFile) {
            throw new Error(`toSource() could only be called on collections of type `
                + `'SourceFile' but this is of type '${ts.SyntaxKind[this.rootNode.kind]}'`);
        }
        return emitter.toSource(this.rootNode);
    }
}
exports.Collection = Collection;
