"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const ts = require("typescript");
function isPatternMatching(node, pattern) {
    if (!matchProperty(node, 'kind', pattern)) {
        return false;
    }
    // tslint:disable-next-line cyclomatic-complexity
    switch (node.kind) {
        case ts.SyntaxKind.Identifier:
            return matchIdentifier(node, pattern);
        case ts.SyntaxKind.CallExpression:
            return matchCallExpression(node, pattern);
        case ts.SyntaxKind.PropertyAccessExpression:
            return matchPropertyAccessExpression(node, pattern);
        case ts.SyntaxKind.FunctionExpression:
            return matchFunctionExpression(node, pattern);
        case ts.SyntaxKind.TypeReference:
            return matchTypeReference(node, pattern);
        default:
            throw new Error(`Pattern for ${ts.SyntaxKind[node.kind]} not implemented`);
    }
}
exports.isPatternMatching = isPatternMatching;
function matchFunctionExpression(_node, _pattern) {
    return true;
}
function matchPropertyAccessExpression(node, pattern) {
    let matching = true;
    matching = matching && matchProperty(node, 'expression', pattern);
    matching = matching && matchProperty(node, 'name', pattern);
    return matching;
}
function matchIdentifier(node, pattern) {
    return matchProperty(node, 'text', pattern);
}
function matchCallExpression(node, pattern) {
    return matchProperty(node, 'expression', pattern);
}
function matchTypeReference(node, pattern) {
    let matching = true;
    matching = matching && matchProperty(node, 'text', pattern);
    return matching;
}
// tslint:disable-next-line cyclomatic-complexity
function matchProperty(node, property, pattern, name = property) {
    const patternValue = pattern[name];
    if (!patternValue) {
        return true;
    }
    const nodeValue = node[property];
    const hasValues = nodeValue && patternValue;
    if (typeof nodeValue === 'string' || typeof nodeValue === 'number') {
        if (nodeValue === patternValue) {
            return true;
        }
        return false;
    }
    return hasValues && isPatternMatching(node[property], pattern[name]);
}
